package dk.nsp.uuid2cpr.proxy.health;

import dk.sosi.seal.vault.CredentialVault;

import java.security.KeyStoreException;
import java.security.cert.Certificate;
import java.security.cert.CertificateExpiredException;
import java.security.cert.CertificateNotYetValidException;
import java.security.cert.X509Certificate;

public class CertificateHealthChecker implements HealthChecker {
    private CredentialVault vault;
    private String alias;

    public CertificateHealthChecker(String alias, CredentialVault vault) {
        this.alias = alias;
        this.vault = vault;
    }

    @Override
    public HealthStatus checkHealth() {
        Certificate certificate = null;
        try {
            certificate = vault.getKeyStore().getCertificate(alias);
        }
        catch(KeyStoreException e) {
            throw new IllegalStateException(String.format("Could not check certificate validity: Certificate not present for alias '%s'.", alias) , e);
        }

        if(!(certificate instanceof X509Certificate)) {
            throw new IllegalStateException(String.format("Could not check certificate validity: Certificate for alias '%s' was not an X509Certificate!", alias));
        }

        X509Certificate x509Certificate = (X509Certificate) certificate;
        try {
            x509Certificate.checkValidity();
        }
        catch (CertificateExpiredException e) {
            return HealthStatus.unhealthy(String.format("Certificate for alias '%s' is expired. Serial number: %d", alias, x509Certificate.getSerialNumber()));
        }
        catch (CertificateNotYetValidException e) {
            return HealthStatus.unhealthy(String.format("Certificate for alias '%s' is not valid yet. Serial number: %d", alias, x509Certificate.getSerialNumber()));
        }

        return HealthStatus.ok();
    }
}
