/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2018 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

package dk.nsp.uuid2cpr.proxy.setup;

import dk.nsp.uuid2cpr.proxy.health.CertificateHealthChecker;
import dk.nsp.uuid2cpr.proxy.health.HealthChecker;
import dk.nsp.uuid2cpr.proxy.ws.ServiceApiImpl;
import dk.nsp.uuid2cpr.proxy.service.OpenApiWriter;
import dk.nsp.uuid2cpr.proxy.service.OpenApiWriterImpl;
import dk.sdsd.nsp.slalog.api.SLALogConfig;
import dk.sdsd.nsp.slalog.api.SLALogger;
import dk.sosi.seal.vault.CredentialVault;
import dk.sosi.seal.vault.FileBasedCredentialVault;
import org.openapitools.api.ServiceApi;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.context.support.PropertySourcesPlaceholderConfigurer;

import java.io.File;
import java.util.Properties;

@Configuration
@PropertySource("classpath:uuid2cpr-proxy.properties")
public class Uuid2CprProxySetup {
	@Bean
	public ServiceApi serviceApi() {
		return new ServiceApiImpl();
	}

	@Bean
	public OpenApiWriter openApiWriter(@Value("${app.endpoint}") String endpoint) {
		return new OpenApiWriterImpl(endpoint);
	}

    @Bean
    public HealthChecker healthChecker(@Autowired CredentialVault credentialVault, @Value("${uuid2cpr.proxy.signature.keystore.alias}") String keystoreAlias) {
        return new CertificateHealthChecker(keystoreAlias, credentialVault);
    }

    @Bean
    public CredentialVault credentialVault(@Value("${uuid2cpr.proxy.signature.keystore.path}") String keystorePath, @Value("${uuid2cpr.proxy.signature.password}") String keystorePassword) {
        return new FileBasedCredentialVault(new Properties(), new File(keystorePath), keystorePassword);
    }

    @Bean
    public SLALogger slaLogger(@Value("${app.name}") String appName, @Value("${app.short.name}") String shortAppName) {
        SLALogConfig slaLogConfig = new SLALogConfig(appName, shortAppName);
        return slaLogConfig.getSLALogger();
    }

	@Bean
	public static PropertySourcesPlaceholderConfigurer propertySourcesPlaceholderConfigurer() {
		return new PropertySourcesPlaceholderConfigurer();
	}
}
