package dk.nsp.uuid2cpr.proxy.ws;

import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sdsd.nsp.slalog.api.SLALogger;
import org.apache.log4j.Logger;
import org.openapitools.api.ServiceApi;

import javax.ws.rs.core.Response;
import javax.ws.rs.core.SecurityContext;
import java.util.Optional;

import dk.sosi.sts.nemlogin.uuid2cpr.Uuid2CprClient;
import dk.sosi.sts.nemlogin.uuid2cpr.Uuid2CprProperties;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

public class ServiceApiImpl implements ServiceApi {
	private final static Logger logger = Logger.getLogger(ServiceApiImpl.class);

    private static final String SLA_LOGPOINT_ID = "1";
    private static final String SLA_LOGPOINT_NAME = "Uuid2Cpr.lookup";

    @Autowired
    private SLALogger slaLogger;

    @Value("${uuid2cpr.proxy.signature.username}")
    private String signatureUsername;

    @Value("${uuid2cpr.proxy.signature.password}")
    private String signaturePassword;

    @Value("${uuid2cpr.proxy.signature.properties.file}")
    private String signaturePropertiesFile;

    @Value("${uuid2cpr.proxy.encryption.username}")
    private String encryptionUsername;

    @Value("${uuid2cpr.proxy.encryption.properties.file}")
    private String encryptionPropertiesFile;

    @Value("${uuid2cpr.proxy.applies.to}")
    private String appliesTo;

    @Value("${uuid2cpr.proxy.security.token.service.url}")
    private String securityTokenServiceUrl;

    @Value("${uuid2cpr.proxy.lookup.service.url}")
    private String lookupServiceUrl;

    @Value("${uuid2cpr.proxy.token.time.to.live}")
    private int tokenTimeToLive;

    @Override
    public Response serviceLookupUuidGet(String uuid, SecurityContext securityContext) {
        logger.info(String.format("Invoking uuid2cpr with uuid %s", uuid));

        Uuid2CprProperties properties = new Uuid2CprProperties();
        properties.setSignatureUsername(signatureUsername);
        properties.setSignaturePassword(signaturePassword);
        properties.setSignaturePropertiesFile(signaturePropertiesFile);
        properties.setEncryptionUsername(encryptionUsername);
        properties.setEncryptionPropertiesFile(encryptionPropertiesFile);
        properties.setAppliesTo(appliesTo);
        properties.setSecurityTokenServiceUrl(securityTokenServiceUrl);
        properties.setLookupServiceUrl(lookupServiceUrl);
        properties.setTokenTimeToLive(tokenTimeToLive);

        Uuid2CprClient client = new Uuid2CprClient(properties);
        SLALogItem item = createSLALogItem(uuid);
        Optional<String> cpr = Optional.empty();
        try {
            cpr = client.lookupCpr(uuid);
            item.setCallResultOk();
        }
        catch(Exception e) {
            item.setCallResultError(e.getMessage());
            throw e;
        }
        finally {
            item.store();
        }

        if(cpr.isPresent()) {
            return Response.ok(ModelMapper.mapToResponseWithDetails(cpr.get())).build();
        }
        return Response.status(Response.Status.NOT_FOUND).entity("Uuid " + uuid + " har ikke noget cpr-nummer tilknyttet.").build();
    }

    private SLALogItem createSLALogItem(String uuid) {
        SLALogItem item = slaLogger.createLogItem(SLA_LOGPOINT_NAME, SLA_LOGPOINT_ID);
        item.addCallParameter("lookupServiceUrl", lookupServiceUrl);
        item.addCallParameter("uuid", uuid);
        return item;
    }
}
