package dk.nsp.uuid2cpr.proxy.ws;

import dk.nsp.uuid2cpr.proxy.health.HealthChecker;
import dk.nsp.uuid2cpr.proxy.health.HealthStatus;
import dk.nsp.uuid2cpr.proxy.service.OpenApiWriter;
import org.apache.http.HttpStatus;
import org.jboss.resteasy.mock.MockHttpRequest;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.MockitoJUnitRunner;
import org.springframework.mock.web.MockHttpServletRequest;
import org.springframework.mock.web.MockHttpServletResponse;

import static org.junit.Assert.*;

@RunWith(MockitoJUnitRunner.class)
public class LookupServletTest {
    @InjectMocks
    private LookupServlet subject;

    @Mock
    private OpenApiWriter openApiWriter;

    @Mock
    private HealthChecker healthChecker;

    @Test
    public void doGet_openapi_returnsDescriptor() throws Exception {
        // Arrange
        MockHttpServletRequest request = new MockHttpServletRequest();
        MockHttpServletResponse response = new MockHttpServletResponse();

        request.setPathInfo("/openapi");

        // Act
        subject.doGet(request, response);

        // Assert
        Mockito.verify(openApiWriter).writeOpenApi(request, response);
    }

    @Test
    public void doGet_health_returns200WhenHealthy() throws Exception {
        // Arrange
        MockHttpServletRequest request = new MockHttpServletRequest();
        MockHttpServletResponse response = new MockHttpServletResponse();

        request.setPathInfo("/health");

        Mockito.when(healthChecker.checkHealth()).thenReturn(HealthStatus.ok());

        // Act
        subject.doGet(request, response);

        // Assert
        assertEquals(HttpStatus.SC_OK, response.getStatus());
    }

    @Test
    public void doGet_health_returns500WhenUnhealthy() throws Exception {
        // Arrange
        MockHttpServletRequest request = new MockHttpServletRequest();
        MockHttpServletResponse response = new MockHttpServletResponse();

        request.setPathInfo("/health");

        String description = "not good";
        Mockito.when(healthChecker.checkHealth()).thenReturn(HealthStatus.unhealthy(description));

        // Act
        subject.doGet(request, response);

        // Assert
        assertEquals(HttpStatus.SC_INTERNAL_SERVER_ERROR, response.getStatus());
        assertTrue(response.getContentAsString().contains(description));
    }

    @Test
    public void doGet_unknownPath_returns404() throws Exception {
        // Arrange
        MockHttpServletRequest request = new MockHttpServletRequest();
        MockHttpServletResponse response = new MockHttpServletResponse();

        request.setPathInfo("/bla");

        // Act
        subject.doGet(request, response);

        // Assert
        assertEquals(HttpStatus.SC_NOT_FOUND, response.getStatus());
    }
}